"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const constants_1 = require("../constants");
const logging_1 = require("../logging");
const clean_1 = require("../utils/clean");
const get_build_tools_installer_path_1 = require("../utils/get-build-tools-installer-path");
const get_python_installer_path_1 = require("../utils/get-python-installer-path");
const get_work_dir_1 = require("../utils/get-work-dir");
const remove_path_1 = require("../utils/remove-path");
const single_line_log_1 = require("../utils/single-line-log");
const launch_1 = require("./launch");
const tailer_1 = require("./tailer");
const singleLineLogger = single_line_log_1.createSingleLineLogger();
const vccInstaller = get_build_tools_installer_path_1.getBuildToolsInstallerPath();
const vcLogTitle = chalk_1.default.bold.green('---------- Visual Studio Build Tools ----------');
const pyLogTitle = chalk_1.default.bold.green('------------------- Python --------------------');
let vccLastLines = ['Still waiting for installer log file...'];
let pythonLastLines = ['Still waiting for installer log file...'];
let lastLinesInterval = null;
const debug = require('debug')('windows-build-tools');
/**
 * Installs the build tools, tailing the installation log file
 * to understand what's happening
 */
function install(cb) {
    logging_1.log(chalk_1.default.green('\nStarting installation...'));
    clean_1.cleanExistingLogFiles();
    remove_path_1.removePath();
    launch_1.launchInstaller()
        .then(() => launchLog())
        .then(() => Promise.all([tailBuildInstallation(), tailPythonInstallation()]))
        .then((details) => {
        cb({ buildTools: details[0], python: details[1] });
    })
        .catch((error) => {
        logging_1.log(error);
    });
}
exports.install = install;
function logStatus() {
    const updatedLog = [vcLogTitle, ...vccLastLines, pyLogTitle, ...pythonLastLines];
    // We expect a length of 16
    if (updatedLog.length < 16) {
        updatedLog.fill('', updatedLog.length, 16);
    }
    if (debug.enabled) {
        updatedLog.forEach((s) => debug(s));
    }
    else {
        singleLineLogger.log(updatedLog.join('\n'));
    }
}
function launchLog() {
    if (!logging_1.shouldLog)
        return;
    logging_1.log('Launched installers, now waiting for them to finish.');
    logging_1.log('This will likely take some time - please be patient!\n');
    logging_1.log('Status from the installers:');
    lastLinesInterval = setInterval(logStatus, 500);
}
function stopLog() {
    if (!logging_1.shouldLog)
        return;
    clearInterval(lastLinesInterval);
    // Flush newlines
    logging_1.log('\n');
}
function tailBuildInstallation() {
    return new Promise((resolve, reject) => {
        const tailer = new tailer_1.Tailer(vccInstaller.logPath);
        tailer.on('lastLines', (lastLines) => {
            vccLastLines = lastLines;
        });
        tailer.on('exit', (result, details) => {
            debug('Install: Build tools tailer exited');
            if (result === 'error') {
                debug('Installer: Tailer found error with installer', details);
                reject(new Error(`Found error with VCC installer: ${details}`));
            }
            if (result === 'success') {
                vccLastLines = [chalk_1.default.bold.green('Successfully installed Visual Studio Build Tools.')];
                debug('Installer: Successfully installed Visual Studio Build Tools according to tailer');
                resolve({ success: true, toConfigure: true });
            }
            // Stop the log now. If we need to report failures, we need
            // to do it after the single-line-logger has stopped messing
            // with the terminal.
            logStatus();
            stopLog();
            if (result === 'failure') {
                logging_1.log(chalk_1.default.bold.red('\nCould not install Visual Studio Build Tools.'));
                logging_1.log('Please find more details in the log files, which can be found at');
                logging_1.log(get_work_dir_1.getWorkDirectory() + '\n');
                debug('Installer: Failed to install according to tailer');
                resolve({ success: false });
            }
        });
        tailer.start();
    });
}
function tailPythonInstallation() {
    return new Promise((resolve, reject) => {
        if (constants_1.IS_PYTHON_INSTALLED) {
            debug('Installer: Python is already installed');
            pythonLastLines = [chalk_1.default.bold.green(`${constants_1.INSTALLED_PYTHON_VERSION} is already installed, not installing again.`)];
            return resolve({ toConfigure: false, installPath: '', success: true });
        }
        // The log file for msiexe is utf-16
        const tailer = new tailer_1.Tailer(get_python_installer_path_1.getPythonInstallerPath().logPath, 'ucs2');
        tailer.on('lastLines', (lastLines) => {
            pythonLastLines = lastLines;
        });
        tailer.on('exit', (result, details) => {
            debug('python tailer exited');
            if (result === 'error') {
                debug('Installer: Tailer found error with installer', details);
                reject(new Error(`Found error with Python installer: ${details}`));
            }
            if (result === 'success') {
                pythonLastLines = [chalk_1.default.bold.green('Successfully installed Python 2.7')];
                debug('Installer: Successfully installed Python 2.7 according to tailer');
                resolve({
                    installPath: details || get_python_installer_path_1.getPythonInstallerPath().targetPath,
                    toConfigure: true,
                    success: true
                });
            }
            if (result === 'failure') {
                logging_1.log(chalk_1.default.bold.red('\nCould not install Python 2.7.'));
                logging_1.log('Please find more details in the log files, which can be found at');
                logging_1.log(get_work_dir_1.getWorkDirectory() + '\n');
                debug('Installer: Failed to install Python 2.7 according to tailer');
                resolve({
                    success: false
                });
            }
        });
        tailer.start();
    });
}
//# sourceMappingURL=index.js.map